@@ -892,6 +892,12 @@ sub collect_one_test_case {
   }
   my @no_combs = grep { $test_combs{$_} == 1 } keys %test_combs;
   if (@no_combs) {
+    if ($::opt_skip_not_found) {
+      push @{$tinfo->{combinations}}, @no_combs;
+      $tinfo->{'skip'}= 1;
+      $tinfo->{'comment'}= "combination not found";
+      return $tinfo;
+    }
     mtr_error("Could not run $name with '".(
         join(',', sort @no_combs))."' combination(s)");
   }
